#ifndef ANIM_H
#define ANIM_H

#include "resources.h"
#include <stdio.h>
#include <vector>
#include <map>
#include <string>
#include <assert.h>
#include "dom.h"

static const int DIRNUM = 8;

class CompositePart
{
public:
	RLE_SPRITE *rle;
	int hotx;
	int hoty;
};

class Frame
{
public:
	int time;
	std::vector<CompositePart> parts;
	void draw(BITMAP *buf, int x, int y);
};

class Anim
{
private:
	std::vector<Frame> frames[DIRNUM];
	static Frame loadFrame (xdom::DomNode &i, Resources *res);
	static CompositePart loadPart (xdom::DomNode &i, Resources *res);
public:
	int totalLength[DIRNUM];
	int totalDirs;
	
	int sizex;
	int sizey;
	int sizez;
	
	Anim();
	~Anim() {}
	
	static void loadFromXml (xdom::DomNode *n, Resources *res, std::map<std::string, Anim*> &result);
	
	void add (Frame &f, int dir);
	
	int getTotalLength(int dir) { return totalLength[dir]; }
	int getTotalDirs() { return totalDirs; }
 	
 	void drawFrame (BITMAP *buf, int f, int dir, int x, int y);
	
	Frame *getFrame (int f, int dir)
	{
		assert (f >= 0 && f < getFrameCount(dir));
		assert (dir >= 0 && dir < DIRNUM);
		return &(frames[dir][f]);
	}

	int getFrameTime (int f, int dir)
	{
		assert (f >= 0 && f < getFrameCount(dir));
		assert (dir >= 0 && dir < DIRNUM);
		return frames[dir][f].time;
	}

	int getFrameCount(int dir)
	{
		assert (dir >= 0 && dir < DIRNUM);
		return frames[dir].size();
	}
	
};

#endif
