#ifndef ENGINE_H
#define ENGINE_H

#include <allegro.h>
#include "button.h"
#include "settings.h"
#include "resources.h"
#include "layer.h"
//#include <aldumb.h>
#include <list>
#include "background.h"
#include "menu.h"
#include "sprite.h"
#include "player.h"
#include "playercontrol.h"
#include "floor.h"
#include "script.h"
#include "insults.h"

using namespace std;

class Main;

class Engine
{
private:
	
	Main *parent;
	
	bool debug;
	
	Settings *settings;
	Resources *resources;
	
	string gameover_message;	
	Main *getParent() { return parent; }
	
	list<Layer*> layers;	
	Background menubg;
	Menu menu;
	
	Floor floor;
	SpriteLayer sprites;
	ViewPort view;
	PlayerControl playerControl;
	Script script;
	Insults insults;
	
	Button btnScreenshot;
	Button btnQuit;
#ifdef DEBUG
	Button btnDebugMode;
#endif

public:	
	Engine (Main *p);
	Resources *getResources() { return resources; }
	void init(); // call once during startup
	
	void startMenu(); // called at startup, after gameover, and after win
	void stopMenu();
	
	void startGame(); // called when game starts
	
	void startScript(); // called after startgame or after you got a level
	void stopScript();

	void startLevel(); // called after script ends
	void stopLevel();
	
	void shutdown(); // called after you quit from menu
	
	void initLevel(int level);
	
	void draw (BITMAP *buffer);
	void update ();
	void spawn();
	
	void done(); // stop music
	bool isDebug () { return debug; }

	int getCounter ();
	int getMaxZOrder ();
		
	void playSample (SAMPLE *s);
	void playMusic (const char *id, float volume = 1.0f);
	void stopMusic ();
	
	
	PlayerControl *getPlayerControl() { return &playerControl; } 
	SpriteLayer *getSpriteLayer() { return &sprites; }
	
	Button* getButton();	
};

#endif
