#include "sprite.h"
#include "spritedata.h"

#ifndef PLAYER_H
#define PLAYER_H

class Anim;

const int RELOAD_TIME = 20;
const int LIVE_TIME = 40;
const int MISSILE_LIVE_TIME = 100;
const float PLAYER_SPEED_INC = 0.1;
const float PLAYER_MAX_SPEED = 5.0;

float sq_dist (Sprite *src, Sprite *dest);
void delta_vector (Sprite *src, Sprite *dest, float &sq_dist, float &radians);

/* makes angle go between -M_PI and M_PI */
void normalize_angle (float &angle);

// base class for anything that can shoot, move, has gravity, explode, etc.
class ActiveSprite : public Sprite
{
public:
	float speed;
	ActiveSprite (Engine *e, SpriteSubType _st);
	int getHealth() { return health; }
	Anim *current;	
	Sprite *target;
	float nozzlex;
	float nozzley;
	float nozzlez;
protected:
	SpriteSubType subType;
	int health;
	void explode (int scale);
	
	Sprite *getTarget () { return target; }

	//vector<Weapon> weapons;
public:
	void setTarget (Sprite *);
	virtual ~ActiveSprite() { setTarget (NULL); } // release lock
	SpriteSubType getSubType() { return subType; }
	
	// fire a weapon from the weapons array
	//void fireWeapon (Weapon & w, Sprite *targe);
	//void inRange (Weapon & w);
	
	void fireBullet (); // for chopper
	void fireBullet2 (); // for enemy bunker
	void fireBullet3 (); // for player tank
	void fireBullet4 (); // for enemy turret / tank
	void fireMissile (); // for sam sites
	
	void draw(BITMAP *buffer, int xofst, int yofst);
	static void init(Resources *res);
};

class Player : public ActiveSprite
{
private:
	bool accelerate;
public:
	Player (Engine *e, SpriteSubType st);
	virtual SpriteType getType () { return ST_PLAYER; } 
	virtual void handleCollission(Sprite *s);
	void setAccelerate();
	virtual void update();	
};

class Enemy : public ActiveSprite
{
private:
	int tReload;
	bool mobile; // mobile yes / no?
	int viewRange; // range where you start shooting
public:
	Enemy (Engine *e, SpriteSubType st);
	virtual SpriteType getType () { return ST_ENEMY; } 
	virtual void handleCollission(Sprite *s);
	virtual void update();
};

class Bonus : public ActiveSprite
{
public:
	Bonus(Engine *e, SpriteSubType st);
	virtual SpriteType getType () { return ST_BONUS; } 
	virtual void handleCollission(Sprite *s) {}
};

class Solid : public ActiveSprite
{
private:
public:
	Solid (Engine *e, SpriteSubType st);
	virtual SpriteType getType () { return ST_OTHER; } 
	virtual void handleCollission(Sprite *s) {}
};

class Bullet : public ActiveSprite
{
private:
	Sprite *source;
	float dx, dy, dz;
	float angle;
	float speed;
	int tLive;
	int maxLive;
	int damage;
	void setSource (Sprite *_source);
public:
	int getDamage() { return damage; }
	Sprite *getSource() { return source; }
	Bullet(Engine *e, Sprite *_source, Sprite *_target, Direction startdir, float _speed);
	Bullet(Engine *e, Sprite *_source, float dx, float dy, float dz);
	virtual SpriteType getType () { return ST_BULLET; } // release lock
	virtual void handleCollission(Sprite *s);
	virtual void update();
	virtual ~Bullet() { setSource (NULL); }
};

class Explosion : public Sprite
{
	int tLive;
	int maxLive;
public:
	Explosion(Engine *e, int size);
	virtual SpriteType getType () { return ST_PLAYER; } 
	virtual void handleCollission(Sprite *s) {}
	virtual void draw(BITMAP *buffer, int xofst, int yofst);
	virtual void update();
};

#endif
