#include "layer.h"
#include <vector>
#include <string>
#include <map>

#ifndef SCRIPT_H
#define SCRIPT_H

using namespace std;

enum Command 
{
	CMD_SAY, // add line "args" in textbox
	CMD_PAUSE, // wait for keypress, then clear
	CMD_WAIT, // wait arg1 msec
	CMD_IMG_CENTER, // get image "args" from resources and display it at center
	CMD_IMG, // get image "args" from resources and display it at arg1, arg2
};

class Instruction
{
public:
	Command cmd;
	int arg1, arg2;
	string args;
};

const int LINEH = 10; // line distance

class Script : public Layer
{
private:
	map <string, vector<Instruction> > scriptmap;
	enum ScriptState { SCRIPT_RUN, SCRIPT_PAUSE, SCRIPT_WAIT_KEYPRESS, SCRIPT_STOP };
	vector<Instruction> currentScript;
	vector<Instruction>::iterator ip;
	ScriptState state;
	int tPause;
	int pause_time;
	int level;
	void runScript();
	
	// objects that are manipulated
	vector <string> lines;
	int textx, texty, textw, texth;
	RLE_SPRITE *img;
	int imgx, imgy;
	
public:
	Script (Engine &e);
	virtual ~Script() {}
	void init();
	void prepare (int level);
	virtual void update();
	virtual void draw(BITMAP *buffer);
	
	void readFromFile (char *filename);
};

#endif
