#ifndef SPRITE_H
#define SPRITE_H

#include <allegro.h>
#include <list>
#include <map>
#include "layer.h"
#include "spritedata.h"

class SpriteLayer;
class Engine;
class Resources;

enum SpriteType { ST_BONUS, ST_BULLET, ST_OTHER, ST_PLAYER, ST_ENEMY };

//NB: we use mathematical order (start at x-axis, ccw)
// but mapped back from isometric to screen coordinates.
enum Direction { SE = 0, S, SW, W, NW, N, NE, E };

const float INV_SQRT2 = 0.7071;

float radiansFromDir (Direction dir);
Direction dirFromRadians (float radians);

class Sprite
{
	friend class SpriteLayer;
private:

	float x, y, z;	
	Direction dir; 
	void setParent (SpriteLayer *_parent);
	
protected:
	
	int color;
	int sizex;
	int sizey;
	int sizez;
	
	bool alive;
	bool visible;
	bool solid;
	bool shadow;
	
	SpriteLayer* parent;
	Engine *engine;
	
	// lock == reference counting mechanism
	void kill () { alive = false; }

public:
	int lock;
	Direction getDir() { return dir; }
	void setDir(Direction _dir);

	Sprite (Engine *e);
	virtual ~Sprite () {}
	
	// only destroy if no references are held
	bool canDestroy() { return (!alive && lock == 0); };
	
	bool isAlive() { return alive; } // if not, scheduled to be killed
	bool isVisible() { return visible; }
	bool isSolid() { return solid; }
	bool hasShadow() { return shadow; }
			
	virtual void drawShadow (BITMAP * buffer, int xofst, int yofst);
	virtual void draw(BITMAP *buffer, int xofst, int yofst);
	
	RLE_SPRITE *getSprite ();
	
	virtual void setLocation (float nx, float ny, float nz);
	virtual void try_move (float dx, float dy, float dz);
		
	float getx () { return x; }
	float gety () { return y; }
	float getz () { return z; }
//	int getSize () { return size; }
	
	virtual void update();
	virtual SpriteType getType () = 0;
	virtual void handleCollission(Sprite *s) = 0;
};

//TODO
const int CELLNUM = 1024;
const int CELLMASK = 1023;
const int CELLSIZE = 512; // cell width / height, must be larger than largest sprite

class SpriteLayer : public Layer
{
private:
	std::list <Sprite*> sprites;
	
	//TODO
	std::list <Sprite*> hashgrid [CELLNUM];
	int getCell (int x, int y);
	
public:
	SpriteLayer (Engine &p) : Layer (p), sprites() {}
	virtual ~SpriteLayer() { killAll(); }
		
	void add(Sprite *o);
	
	void update();
	void killAll();
	void draw (BITMAP *buffer);
	int size() { return sprites.size(); }; // returns # of objects
	
	Sprite * getNearestSprite_ex (
		SpriteType st, 
		bool selectSubType,
		SpriteSubType sst, 
		float x, float y,
		bool selectRange,
		float maxRange, 
		bool selectSpread,
		float range_angle,
		float range_spread);
	
	//Sprite *getNearestSprite (SpriteType st, float x, float y);
	Sprite *getNextBunker ();
	
	void testSprites();
};

#endif
