#include <assert.h>
#include "anim.h"

using namespace xdom;
using namespace std;

Anim::Anim() : frames()
{
	for (int k = 0; k < DIRNUM; ++k)
		totalLength[k] = 0;
	totalDirs = 0;
}

void Anim::add (Frame &f, int dir)
{
	frames[dir].push_back(f);
}

void Anim::loadFromXml (DomNode *n, Resources *res, map<string, Anim*> &result)
{
	assert (n->name == "anim");
	Anim *temp;
	string name = n->attributes["name"];
	vector<DomNode>::iterator i;
	vector<DomNode>::iterator h;
	temp = new Anim;
	temp->totalDirs = 0;
	temp->sizex = atoi (n->attributes["dx"].c_str());
	temp->sizey = atoi (n->attributes["dy"].c_str());
	temp->sizez = atoi (n->attributes["dz"].c_str());
	for (h = n->children.begin(); h != n->children.end(); ++h)
	{
		int length = 0;	
		int dir;
		if (h->name == "all")
		{
			assert (temp->totalDirs == 0); // must be only one "all" tag
			dir = -1;
			temp->totalDirs = 1;
		}
		else if (h->name == "se")
		{
			temp->totalDirs++;
			dir = 0;
		}
		else if (h->name == "s")
		{
			temp->totalDirs++;
			dir = 1;
		}
		else if (h->name == "sw")
		{
			temp->totalDirs++;
			dir = 2;
		}			
		else if (h->name == "w")
		{
			temp->totalDirs++;
			dir = 3;
		}
		else if (h->name == "nw")
		{
			temp->totalDirs++;
			dir = 4;
		}
		else if (h->name == "n")
		{
			temp->totalDirs++;
			dir = 5;
		}
		else if (h->name == "ne")
		{
			temp->totalDirs++;
			dir = 6;
		}
		else if (h->name == "e")
		{
			temp->totalDirs++;
			dir = 7;
		}
		else
		{
			allegro_message ("Unknown tag %s", h->name.c_str());
			assert (false); // unknown tag
		}
		
		for (i = h->children.begin(); i != h->children.end(); ++i)
		{
			Frame frame = loadFrame(*i, res);
			length += frame.time;
			if (dir == -1)
			{
				for (int k = 0; k < DIRNUM; ++k)
					temp->frames[k].push_back(frame);
			}
			else
				temp->frames[dir].push_back(frame);
		}
		if (dir == -1)
		{
			for (int k = 0; k < DIRNUM; ++k)
				temp->totalLength[k] = length;
		}
		else
			temp->totalLength[dir] = length;
	}
	assert (temp->totalDirs == 1 || temp->totalDirs == DIRNUM); 
	// either "all"  (meaning all the same) or four dirs
	
	result.insert (pair<string, Anim*>(name, temp));
}

Frame Anim::loadFrame (DomNode &i, Resources *res)
{
	Frame result;
	vector<DomNode>::iterator j;
	
	result.time = atoi(i.attributes["time"].c_str());
	if (i.name == "frame")
	{
		// just a single part.
		CompositePart part = loadPart (i, res);
		result.parts.push_back (part);
	}
	else if (i.name == "composite")
	{
		for (j = i.children.begin(); j != i.children.end(); ++j)
		{
			// multiple parts
			CompositePart part = loadPart (*j, res);
			result.parts.push_back (part);
		}
	}
	else
	{
		assert (false); // unknown tag
	}
	
	return result;
}

CompositePart Anim::loadPart (DomNode &i, Resources *res)
{
	CompositePart result;
	
	result.rle = res->getRle (i.attributes["rle"]);
	if (!result.rle)
	{
		allegro_message ("error loading animation frame");
	}
	result.hotx = atoi(i.attributes["hotx"].c_str());
	result.hoty = atoi(i.attributes["hoty"].c_str());
	
	return result;
}

void Frame::draw(BITMAP * buf, int x, int y)
{
	vector<CompositePart>::iterator i;
	for (i = parts.begin(); i != parts.end(); ++i)
	{
		draw_rle_sprite (buf, i->rle, x + i->hotx, y + i->hoty);
	}
}

void Anim::drawFrame (BITMAP *buf, int f, int dir, int x, int y) 
{
	assert (f >= 0 && f < getFrameCount(dir));
	assert (dir >= 0 && dir < DIRNUM);
	frames[dir][f].draw(buf, x, y);
}
