#include <assert.h>
#include "settings.h"
#include <allegro.h>
#include <string.h>

const char *config_keys[NUM_BUTTONS] = 
{
	"key_north", "key_south", "key_east", "key_west", "key_northeast", "key_southeast", "key_northwest", "key_southwest", "key_up", "key_down",
	"key_fire", "key_altfire", "key_switch"
};

Settings::Settings()
{
   // these settings are the default
	// they are overriden by the info in xelda.cfg
	// and then by command line parameters
	music = true;
	soundOn = true;
#ifdef DEBUG
	// default in debug version
	windowed = true;
	fpsOn = true;
#else
	// default in release version
	windowed = false;
	fpsOn = false;
#endif
	button[btnN].setScancode (KEY_UP);
	button[btnN].setAltcode (KEY_8_PAD);
	button[btnS].setScancode (KEY_DOWN);
	button[btnS].setAltcode (KEY_2_PAD);
	button[btnE].setScancode (KEY_RIGHT);
	button[btnE].setAltcode (KEY_6_PAD);
	button[btnW].setScancode (KEY_LEFT);	
	button[btnW].setAltcode (KEY_4_PAD);
	button[btnNE].setScancode (KEY_9_PAD);
	button[btnSE].setScancode (KEY_3_PAD);
	button[btnNW].setScancode (KEY_7_PAD);
	button[btnSW].setScancode (KEY_1_PAD);	
	button[btnDown].setScancode (KEY_Z);
	button[btnUp].setScancode (KEY_A);
	button[btnSwitch].setScancode (KEY_TAB);
	button[btnFire].setScancode (KEY_SPACE);	
}

void Settings::getFromConfig()
{
	music = get_config_int ("tins08", "music", music);
	soundOn = get_config_int ("tins08", "sound", soundOn);
	fpsOn = get_config_int ("tins08", "fps", fpsOn);
	windowed = get_config_int ("tins08", "windowed", windowed);
		
	for (int i = 0; i < NUM_BUTTONS; ++i)
	{
		button[i].setScancode (get_config_int ("tins08", config_keys[i], 
			button[i].getScancode()));
	}
}	

void Settings::getFromArgs(int argc, const char *const *argv)
{
		// parse command line arguments
	int i;
	for (i = 1; i < argc; i++)
	{
		if (strcmp (argv[i], "-nosound") == 0)
		{
			soundOn = false;
		}
		else if (strcmp (argv[i], "-windowed") == 0)
		{
			windowed = true;
		}
		else if (strcmp (argv[i], "-fullscreen") == 0)
		{
			windowed = false;
		}
		else if (strcmp (argv[i], "-showfps") == 0)
		{
			fpsOn = true;
		}
		else
		{
			allegro_message (
				"warning: Couldn't parse a command line argument %i\n%s",		
				i, argv[i]);
		}			
	}
}

void Settings::setSoundOn(bool value)	
{
	if (value != soundOn)
	{
		soundOn = value;
	}			
}

void Settings::setMusicOn(bool value)	
{
	if (value != music)
	{
		music = value;
	}			
}
